@@ -5,38 +5,53 @@ use base qw(Exporter);
 use 5.006;
 use strict;
 use warnings;
+use Devel::StackTrace;
 use Carp qw(croak);
 use Data::Dumper qw(Dumper);
 use Fcntl qw(LOCK_EX LOCK_UN);
 use English qw(-no_match_vars);
 
-our $VERSION   = '0.999_002';
-our @EXPORT_OK = qw(to_debug to_debug_raw to_debug_empty);
+our $VERSION = '0.999_003';
+
+our @EXPORT_OK = qw(
+    to_debug
+    to_debug_raw
+    to_debug_empty
+    to_debug_count
+    to_debug_trace
+);
 
 our $HANDLE = \*STDERR;
 
-our $MSG_FORMAT    = qq{%s\n%s\n%s\n%s\n\n};
-our $MSG_DELIMITER = q{-} x 80;
+our $MSG_FORMAT      = qq{%s\n%s\n\n};
+our $MSG_DELIMITER   = q{-} x 80;
+our $MSG_EMPTY_LINES = qq{\n} x 18;
 
 our $DUMPER_INDENT   = 1;
 our $DUMPER_DEEPCOPY = 1;
+our $DUMPER_SORTKEYS = 1;
+
+our %TRACE_PARAMS = (
+    skip_frames => 1, # skip to_debug_trace
+);
+our %TRACE_AS_STRING_PARAMS;
 
 our $COLOR = 'bright_green';
 
-our $MAYBE_COLORED = sub { $ARG[0] };
+our $MAYBE_COLORED = sub { return $ARG[0] };
 eval {
     require Term::ANSIColor;
-    $MAYBE_COLORED = sub { Term::ANSIColor::colored($ARG[0], $COLOR); };
+    $MAYBE_COLORED = sub { return Term::ANSIColor::colored($ARG[0], $COLOR); };
 };
 
 our $MAKE_MSG_HEADER = sub {
     # $MAKE_MSG_HEADER=0, to_debug=1
-    my ($package, undef, $line) = caller(1);
+    my ($package, undef, $line) = caller 1;
 
     # $MAKE_MSG_HEADER=0, to_debug=1, subroutine=2
-    my $subroutine = (caller(2))[3];
+    my $subroutine = (caller 2)[3];
 
-    $subroutine = _resolve_subroutine_name($subroutine);
+    $subroutine = resolve_subroutine_name($subroutine);
 
     my $text = join q{, }, (
         sprintf('package=%s', $package),
@@ -45,9 +60,17 @@ our $MAKE_MSG_HEADER = sub {
     );
     $text = qq{[$text]:};
 
+    {
+        my $msg_delimiter = defined $MSG_DELIMITER ? $MSG_DELIMITER : q{};
+        $text = sprintf qq{%s\n%s\n%s}, $msg_delimiter, $text, $msg_delimiter;
+    }
+
     return $MAYBE_COLORED->($text);
 };
 
+our $NO_LABEL_KEY   = '<no label>';
+our %COUNT_OF_LABEL = ($NO_LABEL_KEY => 0);
+
 # Supported on Perl 5.22+
 eval {
     require Sub::Util;
@@ -56,57 +79,79 @@ eval {
         $set_subname->('MAYBE_COLORED',   $MAYBE_COLORED);
         $set_subname->('MAKE_MSG_HEADER', $MAKE_MSG_HEADER);
     }
-
 };
 
 sub to_debug {
     my $msg_body = do {
         local $Data::Dumper::Indent   = $DUMPER_INDENT;
         local $Data::Dumper::Deepcopy = $DUMPER_DEEPCOPY;
+        local $Data::Dumper::Sortkeys = $DUMPER_SORTKEYS;
 
-        Dumper(\@_);
+        Dumper(\@ARG);
     };
 
-    my $msg_delimiter = defined $MSG_DELIMITER ? $MSG_DELIMITER : q{};
-    $msg_delimiter = $MAYBE_COLORED->($msg_delimiter);
+    my $msg = sprintf $MSG_FORMAT, $MAKE_MSG_HEADER->(), $msg_body;
 
-    my $msg = sprintf $MSG_FORMAT,
-        $msg_delimiter,
-        $MAKE_MSG_HEADER->(),
-        $msg_delimiter,
-        $msg_body;
-
-    _send_to_output($msg);
+    return send_to_output($msg);
 }
 
 sub to_debug_raw {
     my $msg_text = shift;
 
-    my $msg_delimiter = defined $MSG_DELIMITER ? $MSG_DELIMITER : q{};
-    $msg_delimiter = $MAYBE_COLORED->($msg_delimiter);
+    my $msg = sprintf $MSG_FORMAT, $MAKE_MSG_HEADER->(), $msg_text;
 
-    my $msg = sprintf $MSG_FORMAT,
-        $msg_delimiter,
-        $MAKE_MSG_HEADER->(),
-        $msg_delimiter,
-        $msg_text;
-
-    _send_to_output($msg);
+    return send_to_output($msg);
 }
 
 sub to_debug_empty {
-    my $msg_delimiter = defined $MSG_DELIMITER ? $MSG_DELIMITER : q{};
-    $msg_delimiter = $MAYBE_COLORED->($msg_delimiter);
+    my $msg = sprintf $MSG_FORMAT, $MAKE_MSG_HEADER->(), $MSG_EMPTY_LINES;
 
-    my $msg = sprintf qq{%s\n%s%s},
-        $msg_delimiter,
-        $msg_delimiter,
-        qq{\n} x 20;
-
-    _send_to_output($msg);
+    return send_to_output($msg);
 }
 
-sub _send_to_output {
+sub to_debug_count {
+    my ($label, $cb);
+    if (@ARG == 2 && ref $ARG[1] eq 'CODE') {
+        ($label, $cb) = @ARG;
+    }
+    elsif (@ARG == 1 && ref $ARG[0] eq 'CODE') {
+        $cb = $ARG[0];
+    }
+    elsif (@ARG == 1) {
+        $label = $ARG[0];
+    }
+
+    $label = defined $label && length $label ? $label : $NO_LABEL_KEY;
+
+    my $count = $COUNT_OF_LABEL{$label} || 0;
+    $count++;
+    $COUNT_OF_LABEL{$label} = $count;
+
+    my $msg_body = sprintf '%s: %d', $label, $count;
+
+    my $msg = sprintf $MSG_FORMAT, $MAKE_MSG_HEADER->(), $msg_body;
+
+    $cb->($label, $count) if defined $cb;
+
+    return send_to_output($msg);
+}
+
+sub to_debug_trace {
+    my $cb = shift;
+
+    my $trace = Devel::StackTrace->new(%TRACE_PARAMS);
+
+    my $msg_body = $trace->as_string(%TRACE_AS_STRING_PARAMS);
+    $msg_body =~ s{\n+$}{}x;
+
+    my $msg = sprintf $MSG_FORMAT, $MAKE_MSG_HEADER->(), $msg_body;
+
+    $cb->($trace) if defined $cb;
+
+    return send_to_output($msg);
+}
+
+sub send_to_output {
     my $msg = shift;
 
     return unless $HANDLE;
@@ -118,7 +163,7 @@ sub _send_to_output {
     return 1;
 }
 
-sub _resolve_subroutine_name {
+sub resolve_subroutine_name {
     my $subroutine = shift;
 
     return unless defined $subroutine;
@@ -138,13 +183,13 @@ JIP::Debug - provides a convenient way to attach debug print statements anywhere
 
 =head1 VERSION
 
-Version 0.999_002
+Version 0.999_003
 
 =head1 SYNOPSIS
 
-    use JIP::Debug qw(to_debug to_debug_raw to_debug_empty);
+    use JIP::Debug qw(to_debug to_debug_raw to_debug_empty to_debug_count to_debug_trace);
 
-    # The to_debug and to_debug_raw functions print messages to an output stream.
+    # The to_debug and other functions print messages to an output stream.
 
     # For complex data structures (references, arrays and hashes) you can use the
     to_debug(
@@ -159,6 +204,58 @@ Version 0.999_002
     # Prints empty lines
     to_debug_empty();
 
+    # Prints the number of times that this particular call to to_debug_count() has been called
+    to_debug_count();
+
+    # Prints a stack trace
+    to_debug_trace();
+
+=head1 METHODS
+
+=head2 to_debug_count
+
+Logs the number of times that this particular call to C<to_debug_count()> has been called.
+
+This function takes an optional argument C<label>. If C<label> is supplied, this function
+logs the number of times C<to_debug_count()> has been called with that particular C<label>.
+
+C<label> - a string. If this is supplied, C<to_debug_count()> outputs the number of times
+it has been called at this line and with that label.
+
+    to_debug_count('label');
+
+If C<label> is omitted, the function logs the number of times C<to_debug_count()> has been
+called at this particular line.
+
+    to_debug_count();
+
+This function takes an optional argument C<callback>:
+
+    to_debug_count('label', sub {
+        my ($label, $count) = @_;
+    });
+
+or
+
+    to_debug_count(sub {
+        my ($label, $count) = @_;
+    });
+
+=head2 to_debug_trace
+
+Logs a stack trace from the point where the method was called.
+
+This function takes an optional argument C<callback>:
+
+    to_debug_trace(sub {
+        my ($trace) = @_;
+    });
+
+=head1 CODE SNIPPET
+
+    use JIP::Debug qw(to_debug to_debug_raw to_debug_empty to_debug_count to_debug_trace);
+    BEGIN { $JIP::Debug::HANDLE = IO::File->new('/home/my_dir/debug.log', '>>'); }
+
 =head1 SEE ALSO
 
 Debug::Simple, Debuggit, Debug::Easy
